
import Foundation
import CoreData

class CoreDataStack {
  
  var context:NSManagedObjectContext
  var psc:NSPersistentStoreCoordinator
  var model:NSManagedObjectModel
  var store:NSPersistentStore?
  
  init() {
    let bundle = NSBundle.mainBundle()
    let modelURL = bundle.URLForResource("RWTracker", withExtension:"momd")
    model = NSManagedObjectModel(contentsOfURL: modelURL!)!
    
    psc = NSPersistentStoreCoordinator(managedObjectModel:model)
    
    context = NSManagedObjectContext()
    context.persistentStoreCoordinator = psc
    
    let documentsURL = applicationDocumentsDirectory()
    let storeURL = documentsURL.URLByAppendingPathComponent("RWTracker")
    
    let options =
    [NSMigratePersistentStoresAutomaticallyOption: true]
    
    var error: NSError? = nil
    store = psc.addPersistentStoreWithType(NSSQLiteStoreType,
      configuration: nil,
      URL: storeURL,
      options: options,
      error:&error)
    
    if store == nil {
      println("Error adding persistent store: \(error)")
      abort()
    }
  }
  
  func saveContext() {
    var error: NSError? = nil
    if context.hasChanges && !context.save(&error) {
      println("Could not save: \(error), \(error?.userInfo)")
    }
  }

  func mostRecentPost() -> NSManagedObject? {
    let fetch = NSFetchRequest(entityName: "Item")
    fetch.sortDescriptors = [NSSortDescriptor(key: "timestamp", ascending: false)]
    if let results = context.executeFetchRequest(fetch, error: nil) {
      if let item = results.first as? NSManagedObject {
        return item
      }
    }

    return nil
  }

  func mostRecentPostForCategory(category: String) -> NSManagedObject? {
    let fetch = NSFetchRequest(entityName: "Item")
    fetch.predicate = NSPredicate(format: "category like %@", argumentArray: [category])
    fetch.sortDescriptors = [NSSortDescriptor(key: "timestamp", ascending: false)]
    if let results = context.executeFetchRequest(fetch, error: nil) {
      if let item = results.first as? NSManagedObject {
        return item
      }
    }

    return nil
  }

  func applicationDocumentsDirectory() -> NSURL {
    let fileManager = NSFileManager.defaultManager()

    if let url = fileManager.containerURLForSecurityApplicationGroupIdentifier("group.com.razeware.rwtracker") {
      return url
    } else {
      let urls = fileManager.URLsForDirectory(.DocumentDirectory, inDomains: .UserDomainMask) as Array<NSURL>
      return urls[0]
    }
  }
}
